"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parser = require("./parser");
const capabilities_helper_1 = require("./capabilities-helper");
const utils_1 = require("./utils");
class NsCapabilities {
    constructor() {
        this.exceptions = new Array();
        this._projectDir = parser.projectDir;
        this._projectBinary = parser.projectBinary;
        this._pluginRoot = parser.pluginRoot;
        this._pluginBinary = parser.pluginBinary;
        this._appPath = parser.appPath;
        this._port = parser.port;
        this._verbose = parser.verbose;
        this._appiumCapsLocation = parser.appiumCapsLocation;
        this._testFolder = parser.testFolder;
        this._storage = parser.storage;
        this._testReports = parser.testReports;
        this._reuseDevice = parser.reuseDevice;
        this._runType = parser.runType;
        this._isSauceLab = parser.isSauceLab;
        this._ignoreDeviceController = parser.ignoreDeviceController;
        this._appiumCaps = capabilities_helper_1.resolveCapabilities(this._appiumCapsLocation, parser.runType, parser.projectDir);
        this.resolveAppPath();
        this.checkMandatoryCapabiliies();
        this.throwExceptions();
    }
    get projectDir() { return this._projectDir; }
    get projectBinary() { return this._projectBinary; }
    get pluginRoot() { return this._pluginRoot; }
    get pluginBinary() { return this._pluginBinary; }
    get port() { return this._port; }
    get verbose() { return this._verbose; }
    get appiumCapsLocation() { return this._appiumCapsLocation; }
    get appiumCaps() { return this._appiumCaps; }
    get testFolder() { return this._testFolder; }
    get storage() { return this._storage; }
    get testReports() { return this._testReports; }
    get reuseDevice() { return this._reuseDevice; }
    get runType() { return this._runType; }
    get isSauceLab() { return this._isSauceLab; }
    get appPath() { return this._appPath; }
    get ignoreDeviceController() { return this._ignoreDeviceController; }
    set appPath(appPath) { this._appPath = appPath; }
    get device() { return this._device; }
    set device(device) { this._device = device; }
    get emulatorOptions() { return (this._emulatorOptions || "-wipe-data -gpu on"); }
    resolveAppPath() {
        if (this._appPath) {
            this._appiumCaps.app = this._appPath;
        }
        if (!this._appiumCaps.app) {
            this._appiumCaps.app = utils_1.getAppPath(this._appiumCaps.platformName.toLowerCase(), this._runType.toLowerCase());
        }
        console.log("Application full path: " + this._appiumCaps.app);
    }
    checkMandatoryCapabiliies() {
        if (!this.isSauceLab && !utils_1.fileExists(this._appiumCaps.app)) {
            this.exceptions.push("The application folder doesn't exist!");
        }
        if (!this._runType) {
            this.exceptions.push("Missing runType! Please select one from appium capabilities file!");
        }
        if (!this._appiumCaps.platformName) {
            this.exceptions.push("Platform name is missing! Please, check appium capabilities file!");
        }
        if (!this._appiumCaps.platformName) {
            this.exceptions.push("Platform version is missing! Please, check appium capabilities file!");
        }
        if (!this._appiumCaps.deviceName && !this._appiumCaps.udid) {
            this.exceptions.push("The device name or udid are missing! Please, check appium capabilities file!");
        }
    }
    throwExceptions() {
        this.exceptions.forEach(msg => {
            utils_1.logErr(msg, true);
        });
        if (this.exceptions.length > 0) {
            const messagesString = this.exceptions.length > 1 ? "messages" : "message";
            throw new Error(`See the ${messagesString} above and fullfill the conditions!!!`);
        }
    }
}
exports.NsCapabilities = NsCapabilities;
//# sourceMappingURL=ns-capabilities.js.map